// Nodes : node.h
//
// Header file for CNode class
//
// See userguide.pdf for copyright information

#ifndef NODE_H
#define NODE_H

#include <fstream>

using namespace std;

#include "realnum.h"

class CNode;
class CUniverse;
class CUniverseCell;

class CNodeConnection
{
  public:
    CNode *node;
    unsigned dir;
};

class CNode
{
 public:
  typedef enum {store=0,null,wire,not,thrust,fuse,rfuse,lfuse,toggle,delta,
                ndelta,detect,or,runfuse,lunfuse,cross,keypress,pulse,
                rslide,lslide,creator,maj,notnot,push,equal,numtypes} NodeType;

  CNode(RealNum x, RealNum y, RealNum a, RealNum xd, RealNum yd, RealNum ad, RealNum r,NodeType nt);
  CNode(ifstream &is, unsigned &label);
  ~CNode(void);

  bool Move(unsigned iteration);
  void Forces(CNode *n);
  void ConnectionForces(void);
  void FrictionForces(void);
  void Connect(CNode *n);
  void Disconnect(CNode *n);
  void SetNum(int n);

  void Process(CUniverse *u, CUniverseCell *uc);
  void Update(void);

  RealNum DistanceSquared(RealNum x, RealNum y);
  int GetInput(unsigned d, CUniverse *u, CUniverseCell *uc);
  void GetSignalValue(RealNum x, RealNum y, RealNum &r2, int &rval);
  void ApplyForce(RealNum f, RealNum a);

  void Draw(void *param,RealNum x, RealNum y, RealNum w, RealNum h, unsigned options);
  void DrawPostscript(ofstream &os,RealNum x, RealNum y, RealNum w, RealNum h);
  void DrawAnim(ofstream &os);

  void Dump(ofstream &os);
  void Change(NodeType nt, unsigned dir);

  RealNum GetXpos(void) {return xpos;}
  RealNum GetYpos(void) {return ypos;}
  RealNum GetRadius(void) {return radius;}
  CNodeConnection *GetConnection(unsigned j) {return &(connections[j]);}
  int GetOutput(unsigned i) {return output[i];}
 private:
  NodeType type;
  RealNum xpos,ypos,angle;
  RealNum xdelta,ydelta,angledelta;
  RealNum radius;
  RealNum sa,ca;
  int buffer[4];
  int state; // Nodes may use this when they need to - initialised to zero
  unsigned moveiteration;

  CNodeConnection connections[4];
  int output[4];
};

#endif
